<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/08/19
 * Time: 11.30
 */

namespace FiloBlu\Rma\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Catalog\Model\ProductRepository;
use Magento\Quote\Model\Quote\Address\Rate;
use Magento\Quote\Model\QuoteManagement;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Quote\Model\QuoteFactory;
use Psr\Log\LoggerInterface;

class ChangeSizeOrder
{


    /**
     * @var ProductRepository
     */
    protected $productRepository;
    /**
     * @var QuoteManagement
     */
    protected $quoteManagement;

    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;

    /**
     * @var Rate
     */
    protected $shippingRate;

    /**
     * @var StoreRepositoryInterface
     */
    protected $storeRepositoryInterface;
    
    /**
     * @var OrderItemRepositoryInterface
     */
    protected $orderItemRepositoryInterface;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderInterface;

    /**
     * @var QuoteFactory
     */
    protected $quoteFactory;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    const CHANGESIZE_PAYMENT = 'changesizepay';

    /**
     * ChangeSizeOrder constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param ProductRepository $productRepository
     * @param QuoteManagement $quoteManagement
     * @param CustomerRepositoryInterface $customerRepository
     * @param Rate $shippingRate
     * @param StoreRepositoryInterface $storeRepositoryInterface
     * @param OrderItemRepositoryInterface $orderItemRepositoryInterface
     * @param OrderRepositoryInterface $orderInterface
     * @param QuoteFactory $quoteFactory
     * @param LoggerInterface $logger
     */
    public function __construct(
                ScopeConfigInterface $scopeConfig,
                ProductRepository $productRepository,
                QuoteManagement $quoteManagement,
                CustomerRepositoryInterface $customerRepository,
                Rate $shippingRate,
                StoreRepositoryInterface $storeRepositoryInterface,
                OrderItemRepositoryInterface $orderItemRepositoryInterface,
                OrderRepositoryInterface $orderInterface,
                QuoteFactory $quoteFactory,
                LoggerInterface $logger
        ) 
    {
        $this->productRepository = $productRepository;
        $this->quoteManagement = $quoteManagement;
        $this->customerRepository = $customerRepository;
        $this->shippingRate = $shippingRate;
        $this->storeRepositoryInterface = $storeRepositoryInterface;
        $this->orderItemRepositoryInterface = $orderItemRepositoryInterface;
        $this->orderInterface = $orderInterface;
        $this->scopeConfig = $scopeConfig;
        $this->quoteFactory = $quoteFactory;
        $this->logger = $logger;
    }


    /**
     * @param array $orderData
     * @return bool|\Magento\Framework\Model\AbstractExtensibleModel|\Magento\Sales\Api\Data\OrderInterface|object|null
     */
    public function createMageOrder(array $orderData)
    {
        
        error_log(__METHOD__);
        $order = $orderData['order'];

        try
        {

            // get original order store
            $orderStore = $order->getStoreId();
            $store = $this->storeRepositoryInterface->getById($orderStore);
            $website_id = $store->getWebsiteId();
            
            // init quote
            $quote = $this->quoteFactory->create();
            $quote->setStore($store);

            // if you have already buyer id then you can load customer directly
            if(isset($orderData['order']['customer_is_guest']) && $orderData['order']['customer_is_guest'] == "1"){
                //for guest order
                $quote->setCustomerIsGuest(true);
                $quote->setCustomerFirstname($order->getBillingAddress()->getFirstName());
                $quote->setCustomerLastname($order->getBillingAddress()->getLastName());
                $quote->setCustomerEmail($order->getCustomerEmail());
            }else{
                $_customer = $this->customerRepository->get($order->getCustomerEmail());
                            $quote->assignCustomer($_customer); //Assign quote to customer
            }

           // $cart->setCurrency();
            $quote->setCurrency();

            // add item to quote
            $item = $orderData['items'];
            $product = $this->productRepository->getById($item['product_id']);
            $order_item = $this->orderItemRepositoryInterface->get($item['original_order_item_id']);
            $final_price = $order_item['original_price'];//($order_item->getRowTotalInclTax() - $order_item->getBaseDiscountAmount()) / $order_item->getQtyOrdered();
            
            $product->setStoreId($orderStore)->setPrice($final_price);
            $quote->addProduct($product,1);

            // Set Address to quote
            $quote->getBillingAddress()->addData( (array) $order->getBillingAddress()->getData());
            $quote->getShippingAddress()->addData( (array) $order->getShippingAddress()->getData());
            
            // Collect Rates and Set Shipping & Payment Method
            $this->shippingRate->setCode('freeshipping_freeshipping')->getPrice(0);
            $shippingAddress = $quote->getShippingAddress();
            
            $shippingAddress->setCollectShippingRates(true)
                ->collectShippingRates()
                ->setShippingMethod('freeshipping_freeshipping');

            $quote->getShippingAddress()->addShippingRate($this->shippingRate);
            $quote->setPaymentMethod(self::CHANGESIZE_PAYMENT); //payment method
            $quote->save();

            $parentSkuReplaced = $orderData['parent_product_sku'];

            $additionalInfo = [
                "parent_order_id" => $order->getId(),
                "parent_order_increment_id" => $order->getIncrementId(),
                "rma_id" => $orderData['rma_id'],
                "rma_entity_id" => $item['rma_entity_id'],
                "parent_order_item_id" => $item['original_order_item_id'],
                "parent_product_sku" => $parentSkuReplaced,
                "grand_total" => $final_price,
                "base_grand_total" => $final_price
            ];

            $quote->getPayment()->setAdditionalInformation($additionalInfo);
            $quote->getPayment()->importData(['method' => self::CHANGESIZE_PAYMENT]);
            $quote->save();

            //prepare quote data according to parent order
            $quoteItem = $quote->getAllItems()[0];
            $originalItem = $orderData['order']->getItems()[$orderData['items']['original_order_item_id']];

            $quoteItem->setPrice($originalItem->getPrice());
            $quoteItem->setBasePrice($originalItem->getBasePrice());
            $quoteItem->setTaxAmount((float)$originalItem->getTaxAmount()/(int)$originalItem->getQtyOrdered());
            $quoteItem->setRowTotal((float)$originalItem->getRowTotal()/(int)$originalItem->getQtyOrdered());
            $quoteItem->setBaseTaxAmount((float)$originalItem->getBaseTaxAmount()/(int)$originalItem->getQtyOrdered());
            $quoteItem->setBaseRowTotal((float)$originalItem->getBaseRowTotal()/(int)$originalItem->getQtyOrdered());
            $quoteItem->setPriceInclTax($originalItem->getPriceInclTax());
            $quoteItem->setBasePriceInclTax($originalItem->getBasePriceInclTax());
            $quoteItem->setRowTotalInclTax((float)$originalItem->getRowTotalInclTax()/(int)$originalItem->getQtyOrdered());
            $quoteItem->setBaseRowTotalInclTax((float)$originalItem->getBaseRowTotalInclTax()/(int)$originalItem->getQtyOrdered());
            $quote->getAllItems()[0] = $quoteItem;
            $quote->save();

            $order = $this->quoteManagement->submit($quote);
            $new_order = $this->orderInterface->get($order->getId());
            //setting new_order data according to parent order
            $new_order->setShippingDescription('Metodo di spedizione - Standard');
            $new_order->setSubtotal((float)$originalItem->getPrice());
            $new_order->setSubtotalInclTax((float)$originalItem->getPriceInclTax());
            $new_order->setBaseSubtotalInclTax((float)$originalItem->getPriceInclTax());
            $new_order->setBaseSubtotal((float)$originalItem->getPrice());
            $new_order->setTaxAmount((float)$originalItem->getBaseTaxAmount()/(int)$originalItem->getQtyOrdered());
            $new_order->setBaseTaxAmount((float)$originalItem->getBaseTaxAmount()/(int)$originalItem->getQtyOrdered());
            $new_order->setTotalDue((float)$originalItem->getPriceInclTax());
            $new_order->setBaseTotalDue((float)$originalItem->getPriceInclTax());
            $new_order->setGrandTotal((float)$originalItem->getPriceInclTax());
            $new_order->setBaseGrandTotal((float)$originalItem->getPriceInclTax());
            // set pending status - emmedata sync
            $new_order->setStatus('pending');
            $new_order->save();
            // set change_size status
            $new_order->setStatus(ChangeSizeAttributeHelper::ORDER_STATUS_CHANGESIZE_CODE);
            $new_order->save();
            
            return $order->getId();
        }
        catch (\Exception $e)
        {
            $this->logger->critical($e->getMessage());
            return false;
        }

    }


}